<?php

/***
 * 
 * k.Bot - modular irc bot
 * Copyright (C) 2004 Kore Nordmann - http://www.kore-nordmann.de (German)
 * 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or (at 
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
 * 
 * You may contact the author by mail:
 * kbot@kore-nordmann.de
 *
 * Or write to:
 * Kore Nordmann
 * Stockumer Strasse 404
 * 44227 Dortmund
 * Germany
 *
 * The latest version of k.Bot can be obtained from:
 * http://www.kore-nordmann.de/
 *
 **/

// Einbinden benoetigter Klassen
require_once(dirname(__FILE__) . '/irc.php');
require_once(dirname(__FILE__) . '/mysql.php');

class ircBot {
	
	private $id;			// ID des Bots in der DB
	private $mysql;			// MySQL-Klasse
	private $irc;			// IRC-Klasse
	private $events;		// Zeitplan mit anstehenden Events
	private $channel;		// List der Channel, in denen der Bot aktiv ist
	
	/**
	* @return ircBot
	* @param int 	$id		ID des Bots in der DB
	* @desc Konstruktor des ircBot
	*/
	public function __construct(		$id) {
		$this->id = intval($id);
		$this->mysql = new mysql();
		$this->events = array();
		$this->channel = array();
		
		// Basisdaten aus der Datenbank holen
		if (!$this->mysql->query('SELECT nick, ident, name FROM bot WHERE id = ' . $this->id)) die('Ungueltige Bot-ID uebergeben');
		$result = reset($this->mysql->getAll());
		$this->irc = new irc($result['nick'], $result['ident'], $result['name']);
		
		$this->resetDB();

		// Server einfuegen
		$this->mysql->query('SELECT host, port FROM server WHERE bot_id = ' . $this->id);
		while ($this->mysql->isNext()) {
			$row = $this->mysql->getNext();
			$this->irc->addServer($row['host'], $row['port']);
		}

		// Channel einfuegen
		$this->mysql->query('SELECT id, name FROM channel WHERE bot_id = ' . $this->id . '');
		$this->channel = array();
		while ($this->mysql->isNext()) {
			$row = $this->mysql->getNext();
			$this->channel[$row['name']] = intval($row['id']);
		}
	}
	
	/**
	* @return void
	* @desc Liesst die Einstellungen aus der Datenbank und setzt sie fuer den Bot um
	*/
	private function refreshBotData() {
		
		$this->irc->log('Lese Einstellungen neu ein.');
		
		// Speichern des Logs in der DB
		$this->mysql->query('DELETE FROM log WHERE bot_id = ' . $this->id);
		$this->mysql->insert('log', array('bot_id' => $this->id, 'log' => $this->irc->showLog()));
		
		// Channel ueberpruefen
		$this->mysql->query('SELECT id, name, active FROM channel WHERE bot_id = ' . $this->id . '');
		$channel = '';
		$oldChannel = $this->channel;
		$this->channel = array();
		while ($this->mysql->isNext()) {
			$row = $this->mysql->getNext();
			if (($row['active'] == '0') && ($row['name'] != '*')) $channel .= $row['name'] . ',';
			$this->channel[$row['name']] = intval($row['id']);
		}
		if ($channel) $this->irc->join(substr($channel, 0, -1));

		// Ehemalige Channel verlassen
		foreach ($oldChannel as $channel => $id) {
			if (!isset($this->channel[$channel])) {
				$this->irc->part($channel);
				$this->mysql->query('DELETE FROM user WHERE channel_id = ' . $id);
			}
		}
		
		// Nick ueberpruefen
		if ($this->mysql->query('SELECT nick FROM bot WHERE id = ' . $this->id)) {
			$this->irc->setNick($this->mysql->getSingle());
		}
		
		// Naechste Ueberpruefung setzen
		$this->irc->log('Setze naechsten Refresh fuer ' . date('H:i.s', time() + KBOT_REFRESH) . ' an.');
		$this->addTimedEvent('+' . KBOT_REFRESH, 'refresh:()');
	}
	
	/**
	* @return void
	* @desc Setzt die Datenbank zurueck
	*/
	private function resetDB() {
		// Alle Channel und Bot als inaktiv setzen
		$this->irc->log('Setze Datenbank zurueck.');
		$this->mysql->update('channel', array('active' => 0), 'bot_id = ' . $this->id);
		$this->mysql->update('bot', array('active' => 0), 'id = ' . $this->id);
		$this->mysql->query('DELETE FROM user WHERE bot_id = ' . $this->id);
	}
	
	/**
	* @return boolean
	* @param int	$time		Timestamp
	* @param string $event		Event
	* @desc Fuegt einen "Event" zur zeitversetzten Ausfuehrung in den Zeitplan ein
	*/
	private function addTimedEvent(		$time,
										$event) {
		if (is_int($time) && $time >= time()) {
			$this->events[intval($time)] = $event;
		} elseif ($time{0} == '+') {
			$this->events[time() + intval(substr($time, 1))] = $event;
		} else {
			return false;
		}
		
		ksort($this->events);
		return true;
	}
	
	/**
	* @return void
	* @desc Fuehrt anstehende Ereignisse im zeitplan aus
	*/
	private function handleTimedEvents() {
		foreach ($this->events as $time => $event) {
			if ($time > time()) break;
			
			unset($this->events[$time]);
			$this->handleEvent($event);
		}
	}
	
	/**
	* @return string
	* @param string $url	basisurl
	* @param array	$array	GET-Parameter
	* @desc Erstellt eine URL aus der Basisurl mit den angegebenen GET-Parametern
	*/
	private function getURL(			$url,
										$array) {
		$url .= '?';
		foreach ($array as $key => $value) {
			$url .= rawurlencode($key) . '=' . rawurlencode($value) . '&';
		}
		return substr($url, 0, -1);
	}
	
	/**
	* @return boolean
	* @param string	$event	Event
	* @desc Verarbeitet ein Ereignis, bzw.. setzt es in die entsprechenden IRC-Kommandos um
	*/
	private function handleEvent(		$event) {
		if (!preg_match('/^\s*(msg|notice|timed|join|part|kick|mode|unban|ban|nick|quit|refresh)\s*:?\s*([^:\(]*)\s*:?\s*([^:\(]*)\s*\(?(.*?)\)?\s*$/iS', $event, $match)) {
			$this->irc->log('Kaputte Eingabe: ' . $event);
			return false;
		}
		switch (strtolower($match[1])) {
			case 'msg': // Nachricht
				$this->irc->msg($match[2], $match[4]);
			break;
			case 'notice': // Notice
				$this->irc->notice($match[2], $match[4]);
			break;
			case 'timed': // zeitversetzte Ausfuehrung eines Kommandos
				$this->addTimedEvent($match[2], $match[4]);
			break;
			case 'join': // Joinen eines Channels
				$this->irc->join($match[2]);
			break;
			case 'part': // Verlassen eines Channels
				$this->irc->part($match[2]);
			break;
			case 'kick': // Kicken eines Benutzers
				$this->irc->kick($match[2], $match[3], $match[4]);
			break;
			case 'mode': // Aendern der Modi eines Benutzers
				// Diverse Checks einbauen
				$this->irc->mode($match[2], $match[3], $match[4]);
			break;
			case 'ban': // Einem Benutzer bannen
				$this->irc->mode($match[2], '+b', $match[4]);
			break;
			case 'unban': // Ban eines Bneutzers aufheben
				$this->irc->mode($match[2], '-b', $match[4]);
			break;
			case 'nick': // Den Nick aendern
				$this->irc->nick($match[4]);
			break;
			case 'quit': // Den IRC-Server verlassen
				$this->irc->quit($match[4]);
			break;
			case 'refresh': // Daten neu einlesen
				$this->refreshBotData();
			break;
		}
		return true;
	}
	
	/**
	* @return void
	* @param array	$signal		Signalarray
	* @param string	$type		Kommandotyp
	* @param string $command	Kommando
	* @desc Prueft die Berechtigung fuer das Kommando, ruft das entsprechende Modul auf und fuehrt die zurueckgegebenen Events aus
	*/
	private function command(			$signal,
										$type,
										$command = '') {
		// Es wurde zum Bot selbst gesprochen
		if (@$signal['channel'] === $this->irc->getActualNick()) $signal['channel'] = '*';

//		$this->irc->log($type . ':' . $command . ' (' . $signal['channel'] . ':' . @$this->channel[$signal['channel']] . ')');
		
		// Existiert dieses Kommando, dann Rechte holen
		
		/**
		 * Beschreibung des Queries
		 * 
		 * Primaer muss der Query das benoetigte Level fuer das Modul holen, 
		 * dazu wird ein Select ueber die access-Tabelle ausgefuehrt, in der 
		 * die notwedigen berechtigungen stehen. Die Channel-ID fuer die 
		 * Access-Tabelle kommt aus dem vom Bot verwalteten Array und die 
		 * Command-ID wird anhand des Namens und Typs des Kommandos aus der 
		 * Command-Tabelle bezogen.
		 *
		 * Aus der User-Tabelle wird dann, falls der Benutzer dort anwesend 
		 * ist, der Status des Benutzers ausgelesen. Zusaetzlich wird der 
		 * Status im Channel '*' (Botquery) ausgelesen, falls sich der 
		 * Benutzer dort identifiziert hat, gilt der jeweils hoehere Status 
		 * (GREATEST). Wenn der Benutzer nirgendwo aufgefuehrt ist, dann 
		 * wird der Status -1 zurueckgegeben.
		 */
		if (@isset($this->channel[$signal['channel']]) && $this->mysql->query('
		SELECT 
			GREATEST(	IFNULL(u.level, -1),
						IFNULL(us.level, -1))
				AS userlevel,
			a.level AS modullevel,
			m.address 
		FROM 
			access AS a, 
			command AS c, 
			module AS m
		LEFT JOIN 
			user AS u
		ON 
				u.nick = \'' . mysql_escape_string($signal['nick']) . '\' 
			AND u.channel_id = ' . $this->channel[$signal['channel']] . ' 
			AND u.level >= a.level
		LEFT JOIN
			user AS us
		ON
				us.nick = \'' . mysql_escape_string($signal['nick']) . '\' 
			AND	us.channel_id = ' . $this->channel['*'] . '
			AND us.level >= a.level
		
		WHERE 
				a.channel_id = ' . $this->channel[$signal['channel']] . ' 
			AND a.command_id = c.id 
			AND c.command = \'' . mysql_escape_string($command) . '\' 
			AND c.type=\'' . $type . '\' 
			AND c.module_id = m.id
		')) {
			
			while ($this->mysql->isNext()) {
				$row = $this->mysql->getNext();
				
				// JEDEN Modulaufruf loggen
//				$this->irc->log($type . ':' . $command . ':' . $row['address']);
				
				// Wenn nichtidentifzierter User etwas abruft, muss modullevel 0 sein
				if (($row['userlevel'] === '-1') && (intval($row['modullevel']) > 0)) break;
				
				// Userlevel an das Modul mituebergeben
				$signal['level'] = max(0, $row['userlevel']);
				
				// Channel eventuell korrigieren
				if ($signal['channel'] === '*') {
					$signal['output'] = $signal['nick'];
				} else {
					$signal['output'] = $signal['channel'];
				}
				
				// Ausgabe holen
				$signal['command'] = $command;
				$signal['self'] = $this->irc->getActualNick();
				$file = file($this->getURL($row['address'], array_slice($signal, 1)));
				foreach ($file as $row) $this->handleEvent(trim($row));
			}
		}
	}
	
	/**
	* @return void
	* @desc Hauptmethode des Bots. Laesst den Bot dauerhaft laufen und verwaltet die Ereignisse
	*/
	public function run() {
		// Zu Beginn, verbinden
		$this->irc->connect();
		// Zehn Sekunden nach dem verbinden die Daten aus der Datenbank lesen und entsprechend handeln
		$this->addTimedEvent('+10', 'refresh:()');
		
		while (true) { // Dauerschleife
		
			// neue Nachrichten lesen
			$this->irc->listen();

			// Eingegangende Signale abarbeiten
			foreach ($this->irc->signals as $signal) {
				
				// Dem Signalarray die Bot-ID hinzufuegen (relevant fuer einige Module)
				$signal['bot'] = $this->id;
				
				switch (reset($signal)) {
					case IRC_CONNECT:
						// Update der Informationen in der Datenbank
						$this->mysql->update('bot', array('active' => 1), 'id = ' . $this->id);
					break;
					case IRC_JOIN:
						// Update der Informationen in der Datenbank
						if ($signal['nick'] == $this->irc->getActualNick()) {
							$this->mysql->update('channel', array('active' => 1), 'id = ' . $this->channel[$signal['channel']]);
						}
						$this->mysql->insert('user', array(	'channel_id' => 	$this->channel[$signal['channel']],
															'bot_id' =>			$this->id,
															'nick' =>			$signal['nick']));
						// Aufruf der Modulbehandlung
						$this->command($signal, 'action', 'join');
					break;
					case IRC_QUIT:
						// Update der Informationen in der Datenbank
						if ($signal['nick'] === $this->irc->getActualNick()) $this->resetDB();
						$this->mysql->query('DELETE FROM user WHERE nick = \'' . mysql_escape_string($signal['nick']) . '\' AND bot_id = ' . $this->id);
						// Aufruf der Modulbehandlung
						$this->command($signal, 'action', 'quit');
					break;
					case IRC_PART:
						// Update der Informationen in der Datenbank
						if (($signal['nick'] === $this->irc->getActualNick()) && isset($this->channel[$signal['channel']])) {
							$this->mysql->update('channel', array('active' => 0), 'id = ' . $this->channel[$signal['channel']]);
						} else {
							$this->mysql->query('DELETE FROM user WHERE nick = \'' . mysql_escape_string($signal['nick']) . '\' AND channel_id = ' . $this->channel[$signal['channel']]);
						}
						// Aufruf der Modulbehandlung
						$this->command($signal, 'action', 'part');
					break;
					case IRC_KICK:
						// Update der Informationen in der Datenbank
						if ($signal['kicked'] === $this->irc->getActualNick()) {
							// Bot wurde gekickt
							$this->irc->log('Wurde aus ' . $signal['channel'] , ' herausgeworfen.');
							$this->mysql->update('channel', array('active' => 0), 'id = ' . $this->channel[$signal['channel']]);
						}
						// Aufruf der Modulbehandlung
						$this->command($signal, 'action', 'kick');
					break;
					case IRC_NICK:
						// Update der Informationen in der Datenbank
						$this->mysql->update('user', array('nick' => $signal['newnick']), 'nick = \'' . mysql_escape_string($signal['nick']) . '\' AND bot_id = ' . $this->id);
						// Aufruf der Modulbehandlung
						$this->mysql->query('SELECT c.name FROM user AS u, channel AS c WHERE u.nick = \'' . mysql_escape_string($signal['newnick']) . '\' AND u.bot_id = ' . $this->id . ' AND c.id = u.channel_id');
						while ($this->mysql->isNext()) {
							$signal['channel'] = reset($this->mysql->getNext());
							$this->command($signal, 'action', 'nick');
						}
					break;
					case IRC_MSG:
						// Aufruf der Modulbehandlung
						$this->command($signal, 'text');
					break;
					case IRC_COMMAND:
						// Aufruf der Modulbehandlung
						$this->command($signal, 'command', $signal['command']);
					break;
					case IRC_GETMODE:
						// Update der Informationen in der Datenbank
						$this->mysql->query('UPDATE `user` SET `level` = `level` | ' . $signal['mode'] . ' WHERE nick = \'' . mysql_escape_string($signal['nick']) . '\' AND channel_id = ' . $this->channel[$signal['channel']]);
						// Aufruf der Modulbehandlung
						$this->command($signal, 'action', 'mode');
					break;
					case IRC_LOSEMODE:
						// Update der Informationen in der Datenbank
						$this->mysql->query('UPDATE `user` SET `level` = `level` & ~' . $signal['mode'] . ' WHERE nick = \'' . mysql_escape_string($signal['nick']) . '\' AND channel_id = ' . $this->channel[$signal['channel']]);
						// Aufruf der Modulbehandlung
						$this->command($signal, 'action', 'mode');
					break;
					default:
						var_dump($signal);
					break;
				}
			}
			// Signale abgearbeitet, also Array leeren
			$this->irc->signals = array();
			
			// Zeitplan ueberpruefen, ob dort etwas abgearbeitet werden muss
			$this->handleTimedEvents();

			// Bis zum naechsten Schleifendurchlauf warten.
			usleep(KBOT_SLEEP);
		}
	}
}

?>