<?php

/***
 * 
 * k.Bot - modular irc bot
 * Copyright (C) 2004 Kore Nordmann - http://www.kore-nordmann.de (German)
 * 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or (at 
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
 * 
 * You may contact the author by mail:
 * kbot@kore-nordmann.de
 *
 * Or write to:
 * Kore Nordmann
 * Stockumer Strasse 404
 * 44227 Dortmund
 * Germany
 *
 * The latest version of k.Bot can be obtained from:
 * http://www.kore-nordmann.de/
 *
 **/

// Datenbankdaten
define('MYSQL_SERVER',		'server');			// MySQL-Server
define('MYSQL_USER',		'user');			// MySQL-Benutzer
define('MYSQL_PASSWORD',	'password');		// Passwort des MySQL-Benutzers
define('MYSQL_DATABASE',	'database');		// Datenbank

/***
 * Klasse zum Handling von MySQL-Connections
 *
 * Diese Klasse bietet Methoden fuer MySQL-Aktionen, oder ein einfacheres
 * Interface.
 *
 * @access	public
 * @version 		1.0
 * @author			Kore Nordmann <mail@kore-nordmann.de>
 */
class mysql {
	/**
	 * Enthaelt die Verbindungskennung zum MySQL-Server
	 * @var		int
	 * @access	private
	 */
	private $conn;
	/**
	 * Enthaelt die Ergebnisskennungen von MySQL-Queries
	 * @var		array
	 * @access	private
	 */
	private $result;
	/**
	 * Enthaelt die noch vorhandenen Ergebnisse der vorhandenen Queries
	 * @var		array
	 * @access	private
	 */
	private $count;
	
	/**
	 * Konstruktor der Klasse
	 *
	 * Initialisiert die Objektvariablen
	 *
	 * @return	object	$this
	 * @access	public
	 * @version 		1.0
	 * @author			Kore Nordmann <mail@kore-nordmann.de>
	 */
	public function __construct() {
		$this->conn = null;
		$this->result = array();
		$this->count = array();
	}
	
	/**
	 * Verbindungsaufbau
	 *
	 * Baut eine Verbindung zum MySQL-Server auf.
	 *
	 * @param	string	$datafile		Dateiname unter dem die Verbindungsdaten zu finden sind
	 * @return	boolean	true, im Erfolgsfall
	 * @access	public
	 * @version 		1.0
	 * @author			Kore Nordmann <mail@kore-nordmann.de>
	 */
	private function connect(
	) {
		if ($this->conn) return true;
		// Versuch des Verbindungsaufbau
		$this->conn = @mysql_connect(MYSQL_SERVER, MYSQL_USER, MYSQL_PASSWORD);
		// Pr�fung, ob der Verbindungsaufbau geklappt hat
		if ($this->conn === false) {
			trigger_error('Kann keine Verbindung zum MySQL-Server aufbauen (' . mysql_error() . ').', E_USER_ERROR);
			return false;
		}
		return $this->selectDB(MYSQL_DATABASE);
	}
	
	/**
	 * Wahl der Datenbank
	 *
	 * Diese Funktion waehlt eine Datenbank aus, in der die Queries durchgefuehrt werden sollen.
	 *
	 * @param	string	$database	Name der Datenbank
	 * @return	void
	 * @access	public
	 * @version 		1.0
	 * @author			Kore Nordmann <mail@kore-nordmann.de>
	 */
	private function selectDB(
	$database
	) {
		if (!@mysql_select_db($database, $this->conn)) {
			trigger_error('Kann Datenbank nicht auswaehlen (' . mysql_error() . ').', E_USER_ERROR);
			return false;
		}
		return true;
	}
	
	/**
	 * Query abschicken
	 *
	 * Diese Funktionen setzt einen Query an die MySQL-Datenbank ab und speichert das
	 * Ergebnis im Erfolgsfall und gibt die Anzahl der selektierten / betroffenen
	 * datens�tze zur�ck.
	 *
	 * @param	string	$query		Query, der abgesetzt werden soll
	 * @param	mixed	$nummer		Eindeutige Identifikationsnummer oder -string, der den Query identifiziert
	 * @return	int		Anzahl der selektierten Datens�tze
	 * @access	public
	 * @version 		1.0
	 * @author			Kore Nordmann <mail@kore-nordmann.de>
	 */
	public function query(
	$query,
	$nummer = 0
	) {
//		echo "$query\n";
		// verbinde, falls noch nicht verbunden
		$this->connect();
		// Query an die Datenbank absetzen
		$this->result[$nummer] = @mysql_query($query, $this->conn);
		// Ist ein Fehler aufgetreten, dann false zur�ckgeben und Fehler speichern
		if ($this->result[$nummer] === false) {
			trigger_error('Fehler beim ausfuehren des MySQL-Queries (' . $query . '): "' . mysql_error() . '"', E_USER_WARNING);
			return false;
		}
		// Anzahl der betroffenen / selektierten Datens�tze erfragen
		$count = @mysql_num_rows($this->result[$nummer]);

		// Wenn keine Datens�tze selektiert, oder betroffen, l�sche Ergebnis aus dem Array
		if (!$count) {
			unset($this->result[$nummer]);
			$this->count[$nummer] = 0;
		} else {
			// Sonst speicher die Anzahl der Datens�tze
			$this->count[$nummer] = $count;	
		}
		
		if (!$count) $count = intval(@mysql_affected_rows($this->result[$nummer]));

		// R�ckgabe der Anzhal
		return $count;
	}
	
	/**
	 * Insertquery absetzen
	 *
	 * Dient dazu eine INSERT-Operation in der Datenbank auszuf�hren, dabei die einzuf�genden
	 * Daten zu escapen, um ein MySQL-Intrusion vorzubeugen, sowohl nach Wunsch die Insert-ID
	 * zur�ckgegeben zu bekommen.
	 *
	 * @param	string	$tabelle	Name der Tabelle, in die eingef�gt werden soll
	 * @param	array	$array		Daten, die Eingef�gt werden sollen (als assoziativer Array Feldname => Inhalt)
	 * @param	mixed	$nummer		Eindeutige Identifikationsnummer oder -string, der den Query identifiziert
	 * @param	boolean	$getID		Soll die InsertID zur�ckgegeben werden?
	 * @return	object	$this
	 * @access	public
	 * @version 		1.0
	 * @author			Kore Nordmann <mail@kore-nordmann.de>
	 */
	public function insert(
	$tabelle,
	$array,
	$getID = true
	) {
		// verbinde, falls noch nicht verbunden
		$this->connect();
		// Escapen der Daten
		foreach ($array as $key => $value) {
			$array[mysql_real_escape_string($key)] = mysql_real_escape_string($value);
			if ($key !== mysql_real_escape_string($key)) unset($array[$key]);
		}
		// Query erstellen
		$query = 'INSERT INTO ' . $tabelle . " (" . implode(", ", array_keys($array)) . ") VALUES ('" . implode("', '", $array) . "')";
		if ($getID) {
			// Query absetzen
			$this->query($query, 'insert');
			// insertID erfragen
			$returnid = @mysql_insert_id($this->conn);
			// R�ckgabe der InserID
			return $returnid;
		} else {
			// Wenn keine InsertID gebraucht wird, dann einfach nur den Query absetzen
			$this->query($query, $nummer);
			return true;
		}
	}

	/**
	 * Updatequery absetzen
	 *
	 * Dient dazu eine UPDATE-Operation in der Datenbank auszuf�hren, dabei die einzuf�genden
	 * Daten zu escapen, um ein MySQL-Intrusion vorzubeugen.
	 *
	 * @param	string	$tabelle	Name der Tabelle, in die eingef�gt werden soll
	 * @param	array	$array		Daten, die Eingef�gt werden sollen (als assoziativer Array Feldname => Inhalt)
	 * @param	string	$condition	Die Bedingung für den Update-Query
	 * @return	object	$this
	 * @access	public
	 * @version 		1.0
	 * @author			Kore Nordmann <mail@kore-nordmann.de>
	 */
	public function update(
	$tabelle,
	$array,
	$condition = '1'
	) {
		// verbinde, falls noch nicht verbunden
		$this->connect();
		// Escapen der Daten
		foreach ($array as $key => $value) {
			$array[] = mysql_real_escape_string($key) . ' = \'' . mysql_real_escape_string($value) . "'";
			unset($array[$key]);
		}
		// Query erstellen
		$query = 'UPDATE ' . $tabelle . ' SET ' . implode(", ", $array) . ' WHERE ' . $condition;
		return $this->query($query, 'update');
	}
	
	/**
	 * Gibt es noch weitere Ergebnisse
	 *
	 * Diese Funktion teilt mit, ob der Query noch weitere Ergebnisse bereith�lt
	 *
	 * @param	mixed	$nummer		Eindeutige Identifikationsnummer oder -string, der den Query identifiziert
	 * @return	boolean	true, wenn weitere Ergebnisse vorhanden sind, sonst false
	 * @access	public
	 * @version 		1.0
	 * @author			Kore Nordmann <mail@kore-nordmann.de>
	 */
	public function isNext(
	$nummer = 0
	) {
		if (!isset($this->count[$nummer]) || ($this->count[$nummer] <= 0)) {
			// Gibt nichts mehr, Speicher leeren
			@mysql_free_result($this->result[$nummer]);
			unset($this->result[$nummer]);
			return false;
		} else {
			return true;
		}
	}
	
	/**
	 * Gibt die n�chste Ergebniszeile zur�ck
	 *
	 * Gibt die n�chste Zeile des MySQL-Ergebnisses als assoziativen Array zur�ck.
	 *
	 * @param	mixed	$nummer		Eindeutige Identifikationsnummer oder -string, der den Query identifiziert
	 * @return	array	Ergebniszeile
	 * @access	public
	 * @version 		1.0
	 * @author			Kore Nordmann <mail@kore-nordmann.de>
	 */
	public function getNext(
	$nummer = 0
	) {
		$this->count[$nummer]--;
		if ($this->count[$nummer] < 0) {
			@mysql_free_result($this->result[$nummer]);
			unset($this->result[$nummer]);
			unset($this->count[$nummer]);
			trigger_error('Versuch weitere MySQL-Ergebniszeilen zu lesen, w�hrend keine mehr existieren.', E_USER_NOTICE);
			return false;
		}
		return @mysql_fetch_assoc($this->result[$nummer]);
	}
	
	/**
	 * Gibt alle Ergebnisse zur�ck
	 *
	 * Gibt alle Ergebnisse des Queries als zweidimensionalen Array in folgenden Form zurueck:
	 *		$array[Zeilennummer][Spalte]
	 *
	 * @param	mixed	$nummer		Eindeutige Identifikationsnummer oder -string, der den Query identifiziert
	 * @return	array	Ergebnis des Queries
	 * @access	public
	 * @version 		1.0
	 * @author			Kore Nordmann <mail@kore-nordmann.de>
	 */
	public function getAll(
	$nummer = 0
	) {
		$i = 0;
		$return = array();
		// ALle Ergebniszeilen auslesen
		while ($return[$i++] = @mysql_fetch_assoc($this->result[$nummer]));
		// Result loeschen
		@mysql_free_result($this->result[$nummer]);
		unset($this->result[$nummer]);
		// Dummyeintrag entfernen
		array_pop($return);
		// Rueckgabe
		return $return;
	}
	
	/**
	* @return mixed
	* @param mixed $nummer
	* @desc Gibt ein enzelnes Ergebnis zurueck
	*/
	public function getSingle(
	$nummer = 0
	) {
		return @reset(@reset($this->getAll($nummer)));
	}
}

?>