<?php

/***
 * 
 * k.Bot - modular irc bot
 * Copyright (C) 2004 Kore Nordmann - http://www.kore-nordmann.de (German)
 * 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or (at 
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
 * 
 * You may contact the author by mail:
 * kbot@kore-nordmann.de
 *
 * Or write to:
 * Kore Nordmann
 * Stockumer Strasse 404
 * 44227 Dortmund
 * Germany
 *
 * The latest version of k.Bot can be obtained from:
 * http://www.kore-nordmann.de/
 *
 **/

/**
 * TicTacToe gegen den Bot spielen
 */

// Installationsanweisungen
if (isset($_GET['install'])) {
	echo "command:tttstart(Startet ein neues TicTacToe-Spiel)\n";
	echo "command:tttturn(Den naechsten Zug machen)\n";
	echo "command:tttshow(Zeigt das Spielfeld des angegebenen Spieles)\n";
	echo "command:tttlist(DListet die noch offenen Spiele auf)\n";
	die();
}

// Einbinden der MySQL-Klasse
require_once(dirname(__FILE__) . '/../classes/mysql.php');

// Schwierigkeitslevel (0 schlagbar, 1 maximal unentschieden)
define('TTT_LEVEL',			1);

/**
 * TicTacToe
 *
 * Klasse, die das SPielfeld, die Speicherung in einer MySQL-DB und die
 * KI uebernimmt (sehr einfache Bewertungsfunktion)
 */

class tictactoe {
	
	private $id;			// Spielnummer
	private $board;			// Spielfeld
	private $player;		// Aktueller Spieler
	private $move;			// Anzahl der getaetigten Zuege
	private $db;			// Dantenbankverbindung
	
	/**
	* @return void
	* @param int	$id		ID des Spieles in der DB
	* @desc Konstruktor
	*/
	public function __construct(		$id = NULL) {
		$this->db = new mysql();
		
		if ($id === NULL) {
			$this->newGame();
			$this->id = NULL;
		} else {
			$this->id = (int) $id;
			$this->loadGame();
		}
	}
	
	/**
	* @return void
	* @desc Ein neues Spiel anlegen
	*/
	private function newGame() {
		$this->board = array(array(0, 0, 0), array(0, 0, 0), array(0, 0, 0));
		$this->player = 1;
	}
	
	/**
	* @return void
	* @desc Spiel in der Datenbank speichern
	*/
	public function saveGame() {
		if ($this->id === NULL) {
			// Spiel neu anlegen, wenn keine ID definiert ist
			$this->id = $this->db->insert('mod_tictactoe', array('board' => serialize($this->board)));
		} else {
			// Datensatz bei definierter ID aktualisieren
			$this->db->update('mod_tictactoe', array('player' => $this->player, 'move' => $this->move, 'board' => serialize($this->board)), 'id = ' . $this->id);
		}
	}
	
	/**
	* @return void
	* @desc Spiel aus der Datenbank laden
	*/
	private function loadGame() {
		if ($this->db->query('SELECT player, move, board FROM mod_tictactoe WHERE id = ' . $this->id)) {
			$row = $this->db->getNext();
			$this->player = (int) $row['player'];
			$this->move = (int) $row['move'];
			$this->board = unserialize($row['board']);
		}
	}
	
	/**
	* @return boolean
	* @param int	$x		X-Koordinate
	* @param int	$y		Y-Koordinate
	* @desc Setzt ein Kreuz an angegebner Position
	*/
	public function turn(				$x, $y) {
		if ($this->move >= 9) return false;
		
		$x = min(2, max($x, 0));
		$y = min(2, max($y, 0));
		
		// Gueltiger Zug?
		if ($this->board[$x][$y] !== 0) return false;
		
		// Setzen
		$this->board[$x][$y] = $this->player;
		if ($this->player === 1) {
			$this->player = 2;
		} else {
			$this->player = 1;
		}
		$this->move++;
		
		return true;
	}
	
	/**
	* @return boolean
	* @param int	$x		X-Koordinate
	* @param int	$y		Y-Koordinate
	* @desc Nimmt einen Zug an den angegebenen Koordinaten zurueck
	*/
	private function undo(				$x, $y) {
		$this->board[$x][$y] = 0;
		if ($this->player === 1) {
			$this->player = 2;
		} else {
			$this->player = 1;
		}
		$this->move--;
		
		return true;
	}
	
	/**
	* @return int
	* @desc Wenn ein Spieler 3 in einer Reihe hat, gibt die Funktion die Spielernumemr zurueck, sonst false
	*/
	private function maked3() {
		// Horizontal
		if (($this->board[0][0] === $this->board[0][1]) && ($this->board[0][0] === $this->board[0][2])) return $this->board[0][0];
		if (($this->board[1][0] === $this->board[1][1]) && ($this->board[1][0] === $this->board[1][2])) return $this->board[1][0];
		if (($this->board[2][0] === $this->board[2][1]) && ($this->board[2][0] === $this->board[2][2])) return $this->board[2][0];
		
		// Vertikal
		if (($this->board[0][0] === $this->board[1][0]) && ($this->board[0][0] === $this->board[2][0])) return $this->board[0][0];
		if (($this->board[0][1] === $this->board[1][1]) && ($this->board[0][1] === $this->board[2][1])) return $this->board[0][1];
		if (($this->board[0][2] === $this->board[1][2]) && ($this->board[0][2] === $this->board[2][2])) return $this->board[0][2];
		
		// Diagonal
		if (($this->board[0][0] === $this->board[1][1]) && ($this->board[0][0] === $this->board[2][2])) return $this->board[0][0];
		if (($this->board[0][2] === $this->board[1][1]) && ($this->board[0][2] === $this->board[2][0])) return $this->board[0][2];

		return false;
	}
	
	/**
	* @return int
	* @param int	$player	Spieler
	* @desc Bewertet die Position des angegebenen Spielers bei dem aktuellen Spielstand
	*/
	private function evaluate(			$player) {
		$eval = 0;
		
		// Wenn einer damit gewonnen hat, entsprechend extrem positiv/negativ werten
		if ($pl = $this->maked3()) {
			if ($pl !== $player) {
				return -1000;
			} else {
				return 1000;
			}
		}
		
		// Feldbewertungen
		if ($this->board[0][0] === $player) $eval += 3;
		if ($this->board[1][0] === $player) $eval += 2;
		if ($this->board[2][0] === $player) $eval += 3;
		if ($this->board[0][1] === $player) $eval += 2;
		if ($this->board[1][1] === $player) $eval += 4;
		if ($this->board[2][1] === $player) $eval += 2;
		if ($this->board[0][2] === $player) $eval += 3;
		if ($this->board[1][2] === $player) $eval += 2;
		if ($this->board[2][2] === $player) $eval += 3;
		
		// Bewertun zurueckgeben
		return $eval;
	}
	
	/**
	* @return array
	* @desc Gibt die Positionen aller moeglichen Spielzuege zurueck
	*/
	private function generate() {
		$array = array();
		
		for ($i = 0; $i < 3; $i++) {
			for ($j = 0; $j < 3; $j++) {
				if ($this->board[$i][$j] === 0) $array[] = array($i, $j);
			}
		}
		
		return $array;
	}
	
	/**
	* @return int
	* @param int	$player	Spieler
	* @param int	$depth	Suchtiefe
	* @desc Bewertet die Stellung des aktuellen Spielers unter angegebener Suchtiefe
	*/
	private function search(			$player,
										$depth = 2) {
		if ($depth === 0) return 0;
		$value = 0;
		
		foreach ($this->generate() as $pos) {
			$this->turn($pos[0], $pos[1]);
			$value += $this->evaluate($player) + $this->search($player, $depth - 1);
			$this->undo($pos[0], $pos[1]);
		}
		return $value;
	}
	
	/**
	* @return boolean
	* @desc Laesst den Computer seinen zug vornehmen
	*/
	public function compute() {
		// Initialisierungen
		$max = -10000;
		$maxpos = array();
		$player = $this->player;
		
		// Zufallspool fuer nicht-entscheidende Zuege
		$pool = array();
		
		// Moegliche Zuege generieren
		$draws = $this->generate();
		if (count($draws)) {
			foreach ($draws as $pos) { // Zuege durchgehen
				
				// Zug bewerten
				$this->turn($pos[0], $pos[1]);
				$value = $this->evaluate($player) + $this->search($player);
				$this->undo($pos[0], $pos[1]);
				
				// Wenn nicht entscheidender Zug, dann in den Pool einfuegen
				if (($value > -500) && ($value < 500)) $pool[] = $pos;
				
				// besten Zug auswaehlen
				if ($value > $max) {
					$max = $value;
					$maxpos = $pos;
				}
			}
			
			// Wenn das Level auf 0 steht nicht den Besten, sondern zufaelligen Zug auswahlen
			if (($max < 500) && count($pool) && !TTT_LEVEL) $maxpos = $pool[array_rand($pool)];
			
			// Ziehen
			return $this->turn($maxpos[0], $maxpos[1]);
		} else {
			return false;
		}
	}
	
	/**
	* @return string
	* @param string	$channel	Channel, in den Ausgegeben werden soll
	* @desc Gibt das brett in IRC-kommandos aus
	*/
	public function ircdump(			$channel) {
		$board = "msg:" . $channel . "(| + 0 1 2 |)\n";
		for ($i = 0; $i < 3; $i++) {
			$board .= "msg:" . $channel . "(| $i ";
			for ($j = 0; $j < 3; $j++) {
				switch ($this->board[$i][$j]) {
					case 1: $board .= 'x '; break;
					case 2: $board .= 'o '; break;
					default: $board .= '. ';
				}
			}
			$board .= "|)\n";
		}
		return $board;
	}
	
	/**
	* @return int
	* @desc Nummer des aktuellen Spielers
	*/
	public function getPlayer() {
		return $this->player;
	}
	
	/**
	* @return int
	* @desc ID des Spieles
	*/
	public function getID() {
		return $this->id;
	}
	
	/**
	* @return string
	* @param srting	$channel	Channel, in den Ausgegeben werden soll
	* @desc Gibt Nachricht, wenn das Spiel zu Ende ist (Wer gewonnen hat, Unentschieden) und loescht das Spiel aus der Datenbank
	*/
	public function win(				$channel) {
		$value = $this->evaluate($this->player);
		if ($value === 1000) {
			$this->db->query('DELETE FROM mod_tictactoe WHERE id = ' . $this->id);
			return "msg:" . $channel . "(Spieler " . $this->player . " hat gewonnen)\n";
		} elseif ($value === -1000) {
			$this->db->query('DELETE FROM mod_tictactoe WHERE id = ' . $this->id);
			return "msg:" . $channel . "(Spieler " . ($this->player == 1 ? 2 : 1) . " hat gewonnen)\n";
		} elseif ($this->move >= 9) {
			$this->db->query('DELETE FROM mod_tictactoe WHERE id = ' . $this->id);
			return "msg:" . $channel . "(Unentschieden)\n";
		}
	}
}

switch ($_GET['command']) {
	case 'tttstart': // Ein neues Spiel starten
	
		// Spiel erstellen
		$ttt = new tictactoe();
		
		// Ausgabe der Startinformationen
		printf("msg:%s(Ein neues TicTacToe-Spiel wurde gestartet. Dein Kreuz machst du mit mit !tttturn %d <x> <y>; Das Spielefeld anzeigen lassen ueber !tttshow %d)\n", $_GET['output'], $ttt->getID(), $ttt->getID());
		
		// Wenn uebergabe stattfand, soll der Bot mit dem Spielen anfangen
		if (!empty($_GET['text'])) {
			$ttt->compute();
			echo $ttt->dump($_GET['output']);
		}
		
		// Speichern des Spieles
		$ttt->saveGame();
	break;
	case 'tttturn': // Ziehen
	
		// Zuginformationen auseinandernehmen
		if (preg_match('/^(\d+)\s+(\d+)\s+(\d+)$/', $_GET['text'], $match)) {
			
			// Starten des entsprechenden Spieles
			$ttt = new tictactoe((int) $match[1]);
			
			// Ziehen, wenn der Zug moeglich ist
			if (!$ttt->turn((int) $match[3], (int) $match[2])) {
				printf("msg:%s(Fehlerhafter Zug (%d))\n", $_GET['output'], __LINE__);
			} else {
				// Ausgabe, ob das Spiel bereits vorbei ist
				echo $ttt->win($_GET['output']);
				
				// Bot zieht
				$ttt->compute();
				
				// Speichern
				$ttt->saveGame();
				
				// Ausgabe
				echo $ttt->dump($_GET['output']);
				echo $ttt->win($_GET['output']);
			}
		} else {
			printf("msg:%s(Ungueltige Angabe eines Zuges; Parameter: <spiel> <x> <y>)\n", $_GET['output']);
		}
	break;
	case 'tttshow': // Ausgabe des angegebenen Spieles
		$ttt = new tictactoe((int) $_GET['text']);
		echo $ttt->dump($_GET['output']);
	break;
	case 'tttlist': // Soll mal die noch offenen Spiele listen
	break;
}

?>